'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _backbone = require('backbone.marionette');

var _backbone2 = _interopRequireDefault(_backbone);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaultModelEvents = 'change';
var defaultCollectionEvents = 'add remove reset';

function defaultGetProps() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      model = _ref.model,
      collection = _ref.collection,
      _ref$state = _ref.state,
      state = _ref$state === undefined ? {} : _ref$state;

  if (model) {
    _underscore2.default.extend(state, model.toJSON());
  }

  if (collection) {
    _underscore2.default.extend(state, { items: collection.toJSON() });
  }

  return state;
}

function extractNewState() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var getProps = options.getProps,
      props = options.props;

  return _underscore2.default.extend(getProps ? getProps(options) : defaultGetProps(options), props);
}

function viewCallback() {
  var _options;

  (_options = this.options).dispatch.apply(_options, arguments);
}

// Override the default remove function in order not to erase the
// Marionette.View from the DOM when updating the the React Component

function unobtrusiveRemove() {
  this.$el.off();
  this.stopListening();
  return this;
}

var ReactBridge = {

  /**
   * function to wrap a React Component in a Marionette View
   *
   * @param {React Component} Component, the react component which will be rendered inside the Marionette View
   * @param {Object} options, configuration for the React Component
   */

  viewFromComponent: function viewFromComponent(Component) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var ReactMarionetteView = _backbone2.default.View.extend({
      onShow: function onShow() {
        if (this._reactInternalInstance) {
          return false;
        }

        // Create and render a container component wrapping the given component

        var Container = function (_React$Component) {
          _inherits(Container, _React$Component);

          function Container() {
            _classCallCheck(this, Container);

            var _this = _possibleConstructorReturn(this, (Container.__proto__ || Object.getPrototypeOf(Container)).call(this));

            _this.state = extractNewState(options);
            return _this;
          }

          _createClass(Container, [{
            key: 'componentDidMount',
            value: function componentDidMount() {
              _underscore2.default.defaults(options, { observe: {} });

              this.initListener(options.model, options.observe.model || defaultModelEvents);
              this.initListener(options.collections, options.observe.collection || defaultCollectionEvents);
            }
          }, {
            key: 'componentWillUnmount',
            value: function componentWillUnmount() {
              this.destroyListener(options.model);
              this.destroyListener(options.collection);
            }
          }, {
            key: 'initListener',
            value: function initListener(entity, events) {
              if (!entity) {
                return;
              }

              if (events instanceof Array) {
                events = events.join(' ');
              }

              entity.on(events, this.updateComponentState.bind(this));
            }
          }, {
            key: 'updateComponentState',
            value: function updateComponentState() {
              var newState = extractNewState(options);
              this.setState(newState);
            }
          }, {
            key: 'destroyListener',
            value: function destroyListener(entity) {
              if (!entity) {
                return;
              }

              entity.off();
            }
          }, {
            key: 'render',
            value: function render() {
              if (_react2.default.isValidElement(Component)) {
                return _react2.default.cloneElement(Component, this.state);
              }

              return _react2.default.createElement(Component, this.state);
            }
          }]);

          return Container;
        }(_react2.default.Component);

        this._reactInternalInstance = _reactDom2.default.render(_react2.default.createElement(Container, null), this.el);
      },
      onDestroy: function onDestroy() {
        this._reactInternalInstance._isMounted = false;
        _reactDom2.default.unmountComponentAtNode(this.el);
      },
      onClose: function onClose() {
        this._reactInternalInstance._isMounted = false;
        _reactDom2.default.unmountComponentAtNode(this.el);
      }
    });

    return new ReactMarionetteView(options);
  },


  /**
   * function to wrap a Marionette View in a React Component
   *
   * @param {Marionette View} MarionetteView, the view which will be wrapped inside the React Component
   * @param {Object} options, configuration for the Marionette View
   */

  componentFromView: function componentFromView(MarionetteView, options) {
    return _react2.default.createClass({
      componentDidMount: function componentDidMount() {
        var parentElem = _reactDom2.default.findDOMNode(this._reactInternalInstance._instance);

        if (MarionetteView instanceof _backbone2.default.View) {
          MarionetteView.setElement(parentElem);
          this._marionetteView = MarionetteView;
          _underscore2.default.extend(this._marionetteView.options, options);
        } else {
          this._marionetteView = new MarionetteView(_underscore2.default.extend({ el: parentElem }, options));
        }

        this._marionetteView.remove = unobtrusiveRemove;
        this.mapEventsToActions(options);
        this._marionetteView.render();
      },
      shouldComponentUpdate: function shouldComponentUpdate() {
        return false;
      },
      componentWillUnmount: function componentWillUnmount() {
        var _this2 = this;

        // Unregister listeners for the user defined marionette events
        _underscore2.default.map(options.eventsToActions, function (action, event) {
          _this2._marionetteView.stopListening(_this2._marionetteView, event);
        });

        this._marionetteView.destroy && this._marionetteView.destroy();
        this._marionetteView.close && this._marionetteView.close();
      },
      createTemplate: function createTemplate(view, opts) {
        var tagName = view instanceof _backbone2.default.View ? view.tagName : view.prototype.tagName;

        var className = view instanceof _backbone2.default.View ? view.className : view.prototype.className;

        if (opts.tagName) {
          tagName = opts.tagName;
        }

        if (opts.className) {
          className = opts.className;
        }

        return _react2.default.createElement(tagName, { className: className }, null);
      },
      delegate: function delegate(methodName) {
        var _marionetteView;

        // Check if method exists
        if (!this._marionetteView[methodName]) {
          return;
        }

        // Pass the arguments to marionette method

        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        return (_marionetteView = this._marionetteView)[methodName].apply(_marionetteView, args);
      },
      mapEventsToActions: function mapEventsToActions(opts) {
        var _this3 = this;

        if (!opts.eventsToActions || _underscore2.default.isEmpty(opts.eventsToActions)) {
          return;
        }

        if (!opts.dispatch || !opts.dispatch instanceof Function) {
          console.error('The `dispatch` function is not defined.');
          return;
        }

        _underscore2.default.map(opts.eventsToActions, function (action, event) {
          _this3._marionetteView.listenTo(_this3._marionetteView, event, function () {
            for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
              args[_key2] = arguments[_key2];
            }

            var myAction = typeof action === 'function' ? action.apply(undefined, args) : action;
            viewCallback.apply(_this3._marionetteView, [myAction].concat(args));
          });
        });

        return this._marionetteView;
      },
      render: function render() {
        return this.createTemplate(MarionetteView, options);
      }
    });
  }
};

exports.default = ReactBridge;
